/////////////////////////////////////////////////////////////////////////////////////////
//rtc_base.h
//Written By Kenichi OHARA, Arai Lab, Osaka University
/////////////////////////////////////////////////////////////////////////////////////////
#ifndef _RTC_BASE_H_
#define _RTC_BASE_H_

#include "rtc_message.h"

/////////////////////////////////////////////////////////////////////////////////////////
//}N`
/////////////////////////////////////////////////////////////////////////////////////////
//*****************************************************
//RTR|[lg̏ԑJڗp
//*****************************************************
#define RTC_UNKNOWN		0x00
#define RTC_CREATED		0x01
#define RTC_STOPPED		0x02
#define RTC_RUNNING		0x03
#define RTC_INACTIVE		0x04
#define RTC_ACTIVE		0x05
#define RTC_ERROR			0x06
#define RTC_RESETTING		0x07

//*****************************************************
//Ԃlp
//*****************************************************
#define RTM_OK           	0x00
#define RTM_ERR          	0x01
#define RTM_FATAL_ERR    	0x02
#define RTM_PARITY_ERR   	0x03

//*****************************************************
//Ackp`
//*****************************************************
#define RTM_ACK_OK         rtm_ack(RTM_OK);
#define RTM_ACK_ERR        rtm_ack(RTM_ERR);
#define RTM_ACK_FATAL_ERR  rtm_ack(RTM_FATAL_ERR);
#define RTM_ACK_PARITY_ERR rtm_ack(RTM_PARITY_ERR);
#define RTC_ACK			0x01

//*****************************************************
//RTC-LitepR}h`
//*****************************************************
//RTC-LitevgRp
#define CMD_LEVEL0 		1
#define CMD_LEVEL1 		2

//Level0pR}h
#define RTC_PROFILE      	0x00
#define RTC_CMD          	0x01
#define INPORT_CMD      	0x02
#define OUTPORT_CMD      	0x03

//Level1pR}h
#define RTC_START        	0x00
#define RTC_STOP         	0x01
#define RTC_RESET        	0x02
#define RTC_ERR_TEST	 	0x03

//*****************************************************
//RTC-Litep^`
//*****************************************************
typedef BYTE RtmRes;
typedef BYTE ComponentState;

/////////////////////////////////////////////////////////////////////////////////////////
//ϐ`
/////////////////////////////////////////////////////////////////////////////////////////
ComponentState gCurrentState 	= RTC_UNKNOWN;
ComponentState gNextState 		= RTC_CREATED;

/////////////////////////////////////////////////////////////////////////////////////////
//֐`
/////////////////////////////////////////////////////////////////////////////////////////
//*****************************************************
//RTR|[lg̃ANeBreB
//*****************************************************
//=====================================================
//ANeBreB{
//=====================================================
RtmRes rtc_on_initialize();
RtmRes rtc_on_startup();
RtmRes rtc_on_shutdown();
RtmRes rtc_on_activate();
RtmRes rtc_on_aborting();
RtmRes rtc_on_error();
RtmRes rtc_on_reset();
RtmRes rtc_on_deactivate();
RtmRes rtc_on_execute();

//=====================================================
//ANeBreBʊ֐
//=====================================================
RtmRes _rtc_state_do(ComponentState state);		//݂̏Ԃێ
RtmRes _rtc_state_entry(ComponentState state);	//VԂ֑J
RtmRes _rtc_state_exit(ComponentState state);	//ʂ̏ԂɑJ

//*****************************************************
//RTC-LiteL̋@\
//*****************************************************
void rtc_main_loop();			//R|[lg̃C[v
void rtc_connect_proxy();
void _rtc_check_err(RtmRes result);
void _rtc_init();				//R|[lg̏
void _rtc_profile();			//vt@C̑M
//=====================================================
//MR}h̉
//=====================================================
void parse_cmd_level0();
void parse_cmd_level1();

//=====================================================
//AckbZ[W̑M
//=====================================================
void rtm_ack(BYTE ret);

//=====================================================
//ԑJڗp֐
//=====================================================
RtmRes _rtc_start();
RtmRes _rtc_stop();
RtmRes _rtc_reset();
RtmRes _rtc_error();


/////////////////////////////////////////////////////////////////////////////////////////
//֐{
/////////////////////////////////////////////////////////////////////////////////////////
//*****************************************************
//vLVւ̐ڑ֐
//*****************************************************
void rtc_connect_proxy(){
	SendMessage("CONNECT");
	_rtc_init();
}

//*****************************************************
//Ԃl̃`FbN֐
//*****************************************************
RtmRes _check_error(RtmRes result){
	//ԂlG[̏ꍇ́CRTC_ERRORɑJ
	if (result == RTM_ERR && gCurrentState == RTC_ACTIVE){
		gNextState = RTC_ERROR;
		return RTM_ERR;
	}
	return RTM_OK;
}

//*****************************************************
//C[v
//*****************************************************
void rtc_mainloop(){
	RtmRes res;
	ComponentState current_state, next_state;
	current_state = gCurrentState;
	next_state    = gNextState;
	gCurrentState = next_state;

	if (current_state != next_state){
		res = _rtc_state_exit(current_state);
		_check_error(res);

		//R|[lgԂ̕ύX
		next_state = gNextState;
		res = _rtc_state_entry(next_state);
		_check_error(res);
	}
	else{
		//ԂێCstate_dos
		res = _rtc_state_do(current_state);
		_check_error(res);
	}
}


//*****************************************************
//RTR|[lg̏ԑJ
//*****************************************************
//=====================================================
//RTR|[lg̏
//=====================================================
void _rtc_init(){
	gCurrentState = RTC_CREATED;
	gNextState    = RTC_STOPPED;
}

//=====================================================
//vt@C̑M(Kvӏ)
//=====================================================
void _rtc_profile(){
	const char *profile = "tSample/Sample component/Generic/0.1/DrSample/COMMUTATIVE/SPORADIC/10/0:DIO:TimedLong:1/0:ADDATA:TimedLong:1\r\n";
	
	SendMessage(profile);
	Delay1ms();
}

//=====================================================
//ԑJڋʊ֐iKvӏj
//=====================================================
RtmRes _rtc_state_do(ComponentState state){
	switch (state){
		case RTC_RUNNING:

		break;

		case RTC_ACTIVE:
      		return rtc_on_execute();
		break;

		case RTC_ERROR:
			return rtc_on_error();
		break;

		case RTC_INACTIVE:

		break;
		
		default:
			return RTM_OK;
		break;
    }
  return RTM_OK;	
}


RtmRes _rtc_state_entry(ComponentState state){
	switch(state){
		case RTC_RUNNING:

		break;
		
		case RTC_ACTIVE:
			return rtc_on_activate();
		break;
		
		case RTC_INACTIVE:
		
		break;
		
		case RTC_RESETTING:
			//Zbg܂ꍇ
			if (rtc_on_reset() == RTM_OK){
				gNextState = RTC_INACTIVE;
				return RTM_OK;
			}
			//Zbg܂Ȃꍇ
			else{
				gNextState = RTC_ERROR;
				return RTM_OK;
			}
		break;
				
		default:
      		return RTM_OK;
      	break;
	}
	return RTM_OK;
}

RtmRes _rtc_state_exit(ComponentState state){
	switch(state){
		case RTC_CREATED:
			return rtc_on_initialize();
		break;
		
		case RTC_STOPPED:
			return rtc_on_startup();
		break;	
	
		case RTC_RUNNING:
			if(gNextState == RTC_STOPPED){
				return rtc_on_shutdown();
				break;
			}
		break;
		
		case RTC_ACTIVE:
			if(gNextState == RTC_ERROR){
				return rtc_on_aborting();
				break;
			}
			else if(gNextState == RTC_INACTIVE){
				return rtc_on_deactivate();
				break;
			}
			return RTM_ERR;
		break;
		
		default:
			return RTM_OK;
			break;
	}
	return RTM_OK;
}

//*****************************************************
//RTC-LiteL̋@\
//*****************************************************
//=====================================================
//MR}h̉ Level0
//=====================================================
void parse_cmd_level0(){
	rtc_message_read();
	if (!rtc_message_parity_check()){
		RTM_ACK_PARITY_ERR;
		return;
	}

	switch (rtc_message_get(CMD_LEVEL0)){
		case RTC_PROFILE:
			SendMessage("Profile");
			_rtc_profile();
		break;
    
		case RTC_CMD:
			parse_cmd_level1();
		break;
    
		case INPORT_CMD:
			rtc_parse_inport_cmd();
      	break;
      	
      	case OUTPORT_CMD:
      		rtc_parse_outport_cmd();
		break;
      	
      	default:
      	break;
    }	
}

//=====================================================
//MR}h̉ Level1
//=====================================================
void parse_cmd_level1(){
	//Level1̃R}h
	switch (rtc_message_get(CMD_LEVEL1)){
		//R|[lg̓Jn
		case RTC_START:
			RTM_ACK_OK;
			_rtc_start();
		break;
		
		//R|[lg̓~
		case RTC_STOP:
			RTM_ACK_OK;
			_rtc_stop();
		break;
		
 		//R|[lg̃Zbg
 		case RTC_RESET:
 			RTM_ACK_OK;
 			_rtc_reset();
 		break;
    
    		//G[Ԃւ̑J
    		case RTC_ERR_TEST:
    			RTM_ACK_OK;
    			_rtc_error();
    		break;
	
		//InAvtiveԂւ̑J
		case RTC_INACTIVE:
			RTM_ACK_OK;
			gNextState = RTC_INACTIVE;
		break;
    
		//ActiveԂւ̑J
		case RTC_ACTIVE:
			RTM_ACK_OK;
			gNextState = RTC_ACTIVE;
		break; 
    
		default:
		break;
    }
}

//=====================================================
//AckbZ[W̑M
//=====================================================
void rtm_ack(BYTE ret){
	rtc_message_begin();			//wb_̑M
	rtc_message_putc(RTC_ACK);		//R}h̑M
	rtc_message_putc(ret);		//R}hsԂ̑M
	rtc_message_end();			//`FbNT̑M
}

//*****************************************************
//ԑJڗp֐
//*****************************************************
//=====================================================
//R|[lg̓Jn
//=====================================================
RtmRes _rtc_start(){
	if(gCurrentState == RTC_STOPPED){ 
     	gNextState = RTC_RUNNING;
     	return RTM_OK;
     }
     else
      	RTM_ACK_ERR;
	return RTM_ERR;
}


//======================================================
//R|[lg̓~
//======================================================
RtmRes _rtc_stop(){
	if(gCurrentState == RTC_ACTIVE){
		gNextState = RTC_STOPPED;
		RTM_ACK_OK;
		return RTM_OK;
	}
	else 
		RTM_ACK_ERR;
	return RTM_ERR;
}


//======================================================
//R|[lg̃Zbg
//======================================================
RtmRes _rtc_reset(){
	if(gCurrentState == RTC_ERROR){
		gNextState = RTC_RESETTING;
		RTM_ACK_OK;
		return RTM_OK;
	}  
	else 
		RTM_ACK_ERR;
	return RTM_ERR;
}

//======================================================
//G[Ԃւ̑J
//======================================================
RtmRes _rtc_error(){
	gNextState = RTC_ERROR;
	RTM_ACK_OK;
	return RTM_OK;
}

#endif
